@@ -845,7 +845,7 @@ void FolderModel::drop(QQuickItem *target, QObject* dropEvent, int row)
         const KDesktopFile file(item.targetUrl().path());
 
         if (file.readType() == QLatin1String("Link")) {
-            dropTargetUrl = file.readUrl();
+            dropTargetUrl = QUrl(file.readUrl());
         } else {
             dropTargetUrl = item.mostLocalUrl();
         }
@@ -1370,12 +1370,7 @@ QList<QUrl> FolderModel::selectedUrls(bool forTrash) const
 
         if (forTrash) {
             // Prefer the local URL if there is one, since we can't trash remote URL's
-            const QString path = item.mostLocalUrl().toString();
-            if (!path.isEmpty()) {
-                urls.append(path);
-            } else {
-                urls.append(item.url());
-            }
+            urls.append(item.mostLocalUrl());
         } else {
             urls.append(item.url());
         }
@@ -1435,6 +1430,7 @@ void FolderModel::moveSelectedToTrash()
     if (uiDelegate.askDeleteConfirmation(urls, KIO::JobUiDelegate::Trash, KIO::JobUiDelegate::DefaultConfirmation)) {
         KIO::Job* job = KIO::trash(urls);
         job->ui()->setAutoErrorHandlingEnabled(true);
+        KIO::FileUndoManager::self()->recordJob(KIO::FileUndoManager::Trash, urls, QUrl(QStringLiteral("trash:/")), job);
     }
 }
 
